using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace PointLight
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Effect pointLightEffect;
        Model model;
        Matrix[] modelTransforms;

        Matrix world;
        Matrix view;
        Matrix projection;

        // This is both the light color and the light intensity 
        Vector3 ambientLightColor;
        // The color of the objects you are going to draw
        Vector3[] diffuseColor;
        // The specular color of the objects
        // The w component stores the specular power
        Vector4[] specularColorPower;
        // The emissive color of the objects
        Vector3[] emissiveColor;
        // The position the light
        Vector3 lightPosition;
        // The range of the light
        float lightRange;
        // The color and intensity of the diffuse light
        Vector3 diffuseLightColor;
        // Specular color of the light
        Vector3 specularLightColor;
        // The position of the camera
        Vector3 cameraPosition;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // We set the camera position
            cameraPosition = new Vector3(0, 1.5f, 3.5f);

            // Set the values for our world, view, and projection matrices
            world = Matrix.Identity;
            view = Matrix.CreateLookAt(cameraPosition, new Vector3(0, 0, 0), new Vector3(0, 1, 0));
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, GraphicsDevice.Viewport.AspectRatio, 0.1f, 100.0f);

            // Set the ambient color
            ambientLightColor = new Vector3(0.05f, 0.1f, 0.2f);

            // Set the diffuse colors
            diffuseColor = new Vector3[4];
            diffuseColor[0] = new Vector3(1, 0.25f, 0.25f);
            diffuseColor[1] = new Vector3(0.25f, 1, 0.25f);
            diffuseColor[2] = new Vector3(0.25f, 0.25f, 1);
            diffuseColor[3] = new Vector3(0.5f, 0.0f, 0.5f);

            // Set the specular color and power
            specularColorPower = new Vector4[4];
            specularColorPower[0] = new Vector4(1, 1, 1, 32.0f);
            specularColorPower[1] = new Vector4(1, 1, 0, 64.0f);
            specularColorPower[2] = new Vector4(0, 1, 1, 32.0f);
            specularColorPower[3] = new Vector4(0, 0, 0, 0);

            // Set the emissive colors
            emissiveColor = new Vector3[4];
            emissiveColor[0] = new Vector3(0, 0, 0);
            emissiveColor[1] = new Vector3(0, 0, 0);
            emissiveColor[2] = new Vector3(0, 0, 0);
            emissiveColor[3] = new Vector3(0, 0, 0);

            // Set light starting location
            lightPosition = new Vector3(0, 2.5f, 2.0f);

            // The range the light
            lightRange = 6.0f;

            // Set the lights diffuse color
            diffuseLightColor = new Vector3(1, 0.9f, 0.8f);

            // Set the lights specular color
            specularLightColor = new Vector3(1, 0.9f, 0.8f);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load custom effect
            pointLightEffect = Content.Load<Effect>("PointLightEffect");

            // Load our model
            model = Content.Load<Model>("LightingModels");

            // Create an array for model transforms and make a copy
            modelTransforms = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(modelTransforms);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            int colorIndex = 0;

            // Rotate and normalize the light
            Matrix rotate = Matrix.CreateRotationY((float)gameTime.ElapsedGameTime.TotalSeconds);
            lightPosition = Vector3.Transform(lightPosition, rotate);

            // Set effect wide parameters
            pointLightEffect.Parameters["View"].SetValue(view);
            pointLightEffect.Parameters["Projection"].SetValue(projection);
            pointLightEffect.Parameters["AmbientLightColor"].SetValue(ambientLightColor);
            pointLightEffect.Parameters["LightPosition"].SetValue(lightPosition);
            pointLightEffect.Parameters["LightRange"].SetValue(lightRange);
            pointLightEffect.Parameters["DiffuseLightColor"].SetValue(diffuseLightColor);
            pointLightEffect.Parameters["SpecularLightColor"].SetValue(specularLightColor);
            pointLightEffect.Parameters["CameraPosition"].SetValue(cameraPosition);

            foreach (ModelMesh mesh in model.Meshes)
            {
                // Set mesh effect parameters
                pointLightEffect.Parameters["World"].SetValue(modelTransforms[mesh.ParentBone.Index] * world);

                foreach (ModelMeshPart meshPart in mesh.MeshParts)
                {
                    // Set vertex and index buffer to use
                    GraphicsDevice.SetVertexBuffer(meshPart.VertexBuffer, meshPart.VertexOffset);
                    GraphicsDevice.Indices = meshPart.IndexBuffer;

                    // Set diffuse color for the object
                    pointLightEffect.Parameters["DiffuseColor"].SetValue(diffuseColor[colorIndex]);
                    pointLightEffect.Parameters["SpecularColorPower"].SetValue(specularColorPower[colorIndex]);
                    pointLightEffect.Parameters["EmissiveColor"].SetValue(emissiveColor[colorIndex]);
                    colorIndex++;

                    // Apply our one and only pass
                    pointLightEffect.CurrentTechnique.Passes[0].Apply();

                    GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0,
                                                         meshPart.NumVertices,
                                                         meshPart.StartIndex,
                                                         meshPart.PrimitiveCount);
                }
            }


            base.Draw(gameTime);
        }
    }
}
