using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;

namespace InvertColorsProcessor
{
    /// <summary>
    /// This class will be instantiated by the XNA Framework Content Pipeline
    /// to apply custom processing to content data, converting an object of
    /// type TInput to TOutput. The input and output types may be the same if
    /// the processor wishes to alter data without changing its type.
    ///
    /// This should be part of a Content Pipeline Extension Library project.
    ///
    /// TODO: change the ContentProcessor attribute to specify the correct
    /// display name for this processor.
    /// </summary>
    [ContentProcessor(DisplayName = "InvertColorsProcessor")]
    public class InvertColorsProcessor : ContentProcessor<Texture2DContent, Texture2DContent>
    {
        public override Texture2DContent Process(Texture2DContent input, 
            ContentProcessorContext context)
        {
            // Convert the input to standard Color format, for ease of processing.            
            input.ConvertBitmapType(typeof(PixelBitmapContent<Color>));            
            foreach (MipmapChain imageFace in input.Faces)            
            {                
                for (int i = 0; i < imageFace.Count; i++)                
                {                    
                    PixelBitmapContent<Color> mip = (PixelBitmapContent<Color>)imageFace[i];                    
                    // Invert the colors
                    for (int w = 0; w < mip.Width; w++)
                    {
                        for (int h = 0; h < mip.Height; h++)
                        {
                            Color original = mip.GetPixel(w, h);
                            Color inverted = new Color(255 - original.R, 
                                255 - original.G, 255 - original.B);
                            if (ShouldCauseBlocks)
                            {
                                if ((h % BlockSize > (BlockSize / 2)) 
                                    || (w % BlockSize > (BlockSize / 2)))
                                {
                                    inverted = original;
                                }
                            }
                            mip.SetPixel(w, h, inverted);

                        }
                    }
                }            
            }            
            return input;
        }

        [DefaultValue(false)]
        [DisplayName("BlockyInversion")]
        [Description("Should the inversion be done in blocks?")]
        public bool ShouldCauseBlocks
        {
            get;
            set;
        }

        [DefaultValue(20)]
        [DisplayName("BlockSize")]
        [Description("The size in pixels of the blocks if a blocky inversion is done")]
        public int BlockSize
        {
            get;
            set;
        }
    }
}