using System;
using System.Threading;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace PerformanceTest
{
    class ModelInformation
    {
        public Model Model;
        public Vector3 Position;
        public Vector3 YawPitchRoll;
        public Vector3 YawPitchRollChange;
        public Vector3 Velocity;
    }
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        const float BoxRange = 200.0f;
        private List<ModelInformation> _modelList;
        Matrix view;
        Matrix proj;
        Random r = new Random();
        int bounceCheck = 0;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.SynchronizeWithVerticalRetrace = false;
            this.IsFixedTimeStep = false;

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            PerformanceHelper.InitializeWithGame(this);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            const int total = 1000;

            _modelList = new List<ModelInformation>();
            for (int i = 0; i < total; i++)
            {
                ModelInformation modelInfo = new ModelInformation();
                modelInfo.YawPitchRoll = GetRandomV3(1.0f);
                modelInfo.Position = GetRandomV3(BoxRange);
                modelInfo.Velocity = GetRandomV3(27.5f);
                modelInfo.YawPitchRollChange = GetRandomV3(5.1f);

                modelInfo.Model = (r.NextDouble() > 0.5) ?
                    Content.Load<Model>("depthmodel") :
                    Content.Load<Model>("environmentmapmodel");

                _modelList.Add( modelInfo );
            }

            view = Matrix.CreateLookAt(new Vector3(0, 3.0f, -250.0f), Vector3.Zero, Vector3.Up);
            proj = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                GraphicsDevice.Viewport.AspectRatio, 1.0f, 1000.0f);

            // turn on some lights
            foreach (ModelMesh mm in Content.Load<Model>("depthmodel").Meshes)
            {
                foreach (BasicEffect be in mm.Effects)
                {
                    be.EnableDefaultLighting();
                }
            }
            foreach (ModelMesh mm in Content.Load<Model>("environmentmapmodel").Meshes)
            {
                foreach (BasicEffect be in mm.Effects)
                {
                    be.EnableDefaultLighting();
                }
            }
            // TODO: use this.Content to load your game content here
        }

        Vector3 GetRandomV3(float max)
        {
            Vector3 v = new Vector3();

            v.X = (float)(r.NextDouble() * max);
            v.Y = (float)(r.NextDouble() * max);
            v.Z = (float)(r.NextDouble() * max);

            if (r.NextDouble() > 0.5)
                v.X *= -1;
            if (r.NextDouble() > 0.5)
                v.Y *= -1;
            if (r.NextDouble() > 0.5)
                v.Z *= -1;

            return v;
        }
        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            PerformanceHelper.StartFrame();
            using (new TimeRulerHelper("Update", Color.Yellow))
            {
                System.Threading.Thread.Sleep(5);

                // Allows the game to exit
                if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                    this.Exit();

                // Update model information
                foreach (ModelInformation m in _modelList)
                {
                    m.Position += m.Velocity * (float)gameTime.ElapsedGameTime.TotalSeconds;
                    m.YawPitchRoll += m.YawPitchRollChange * 
                        (float)gameTime.ElapsedGameTime.TotalSeconds;
                    
                    // Check position
                    if ((m.Position.X <= -BoxRange) || (m.Position.X >= BoxRange) ||
                        (m.Position.Y <= -BoxRange) || (m.Position.Y >= BoxRange) ||
                        (m.Position.Z <= -BoxRange) || (m.Position.Z >= BoxRange)  )
                    {
                        // Move back
                        m.Position -= m.Velocity * 
                            (float)gameTime.ElapsedGameTime.TotalSeconds;
                        m.Velocity *= -1;
                    }

                    // If they're colliding, redirect in some random direction
                    if ((bounceCheck % 10) == 0)
                    {
                        if (IsColliding(m))
                        {
                            m.Velocity *= -GetRandomV3(1.0f);
                            // Start moving that way
                            m.Position += m.Velocity * 
                                (float)gameTime.ElapsedGameTime.TotalSeconds;
                        }
                    }

                    bounceCheck++;
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// Detect if this model is colliding with any other one
        /// </summary>
        /// <param name="m"></param>
        /// <returns></returns>
        private bool IsColliding(ModelInformation m)
        {
            foreach (ModelInformation info in _modelList)
            {
                if (m == info)
                    continue;

                // Calculate difference
                Vector3 difference = info.Position - m.Position;
                float combinedRadius = m.Model.Meshes[0].BoundingSphere.Radius *
                    m.Model.Meshes[0].BoundingSphere.Radius *
                    info.Model.Meshes[0].BoundingSphere.Radius *
                    info.Model.Meshes[0].BoundingSphere.Radius;
                // If they're colliding
                if (difference.LengthSquared() > combinedRadius  )
                {
                    return true;
                }
            }
            return false;
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            using (new TimeRulerHelper("Draw", Color.Blue))
            {
                System.Threading.Thread.Sleep(10);
                GraphicsDevice.DepthStencilState = DepthStencilState.Default;
                foreach (ModelInformation m in _modelList)
                {
                    m.Model.Draw(Matrix.CreateFromYawPitchRoll(m.YawPitchRoll.X,
                        m.YawPitchRoll.Y, m.YawPitchRoll.Z) *
                        Matrix.CreateTranslation(m.Position), view, proj);
                }
            }
            base.Draw(gameTime);
        }
    }
}
