using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.IO;
using Microsoft.Xna.Framework.Storage;

namespace XnaStorage
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        List<Vector2> points;
        List<Vector2> contentPoints;
        SpriteFont font;
        Vector2 currentPos;
        GamePadState lastState;
        StorageDevice storageDevice;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            points = new List<Vector2>();
            currentPos = Vector2.Zero;

            Components.Add(new GamerServicesComponent(this));
            base.Initialize();

            StorageDevice.DeviceChanged += new EventHandler<EventArgs>(
                (o, e) =>
                {
                    // The device changed, make sure my current device is still valid
                    if (storageDevice != null)
                    {
                        if (!storageDevice.IsConnected)
                        {
                            // My device was disconnected, set it to null so I don't try to save anything
                            storageDevice = null;
                            // Warn the user that we won't be saving now
                            Guide.BeginShowMessageBox("No Longer Saving",
                                "The device you were using to save no longer exists " +
                                "so saving automatically has been disabled.",
                                new string[] { "Ok" }, 0, MessageBoxIcon.Warning,
                                new AsyncCallback((result) =>
                                    {
                                        Guide.EndShowMessageBox(result);
                                    }
                                ), null);
                        }
                    }
                }
                );

            // Load the data
            StorageDevice.BeginShowSelector(new AsyncCallback(
                (result) =>
                {
                    storageDevice = StorageDevice.EndShowSelector(result);
                    if (storageDevice != null)
                    {
                        storageDevice.BeginOpenContainer("GameData", new AsyncCallback(
                            (openResult) =>
                            {
                                using (StorageContainer file = 
                                    storageDevice.EndOpenContainer(openResult))
                                {
                                    using (BinaryReader reader = 
                                        new BinaryReader(file.OpenFile("pointData")))
                                    {
                                        for (int i = 0; i < reader.BaseStream.Length / 8; i++)
                                        {
                                            points.Add(new Vector2(reader.ReadSingle(), 
                                                reader.ReadSingle()));
                                        }
                                    }
                                }
                            }
                        ), null);
                    }
                }
            ), null);

            // Read from the loose file
            contentPoints = new List<Vector2>();
            using (StreamReader reader = new StreamReader(
                TitleContainer.OpenStream("data.txt")))
            {
                while (!reader.EndOfStream)
                {
                    string currentLine = reader.ReadLine();
                    string[] numbers = currentLine.Split(' ');
                    contentPoints.Add(new Vector2(float.Parse(numbers[0]),
                        float.Parse(numbers[1])));
                }
            }
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            font = Content.Load<SpriteFont>("font");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }
        protected override void OnExiting(object sender, EventArgs args)
        {
            // Save our data
            if (storageDevice != null)
            {
                storageDevice.BeginOpenContainer("GameData", new AsyncCallback(
                    (openResult) =>
                    {
                        using (StorageContainer file =
                            storageDevice.EndOpenContainer(openResult))
                        {
                            using (BinaryWriter writer =
                                new BinaryWriter(file.CreateFile("pointData")))
                            {
                                foreach (Vector2 v in points)
                                {
                                    writer.Write(v.X);
                                    writer.Write(v.Y);
                                }
                            }
                        }
                    }
                ), null);
            }
            base.OnExiting(sender, args);
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            GamePadState state = GamePad.GetState(PlayerIndex.One);
            if (state.IsButtonUp(Buttons.A) && (lastState.IsButtonDown(Buttons.A)))
            {
                // Add a new point here
                points.Add(currentPos);
            }

            // Move the current cursor
            currentPos.X += (state.ThumbSticks.Left.X * 10);
            currentPos.Y -= (state.ThumbSticks.Left.Y * 10);
            // Don't let it go beyond the screen
            if (currentPos.X < 0)
                currentPos.X = 0;
            if (currentPos.Y < 0)
                currentPos.Y = 0;
            if (currentPos.X > GraphicsDevice.Viewport.Width)
                currentPos.X = GraphicsDevice.Viewport.Width;
            if (currentPos.Y > GraphicsDevice.Viewport.Height)
                currentPos.Y = GraphicsDevice.Viewport.Height;

            lastState = state;
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            spriteBatch.Begin();
            foreach (Vector2 position in points)
            {
                spriteBatch.DrawString(font, "x", position, Color.White);
            }
            foreach (Vector2 position in contentPoints)
            {
                spriteBatch.DrawString(font, "x", position, Color.Black);
            }

            // Draw the cursor, slightly bigger
            spriteBatch.DrawString(font, "*", currentPos, Color.Red);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
